#!/usr/bin/perl -w
################################################################################
# Copyright 2010 Open - http://www.open-groupe.com/
# BMC Service Desk Express Nagios Connector is developed by : Herve Roux under GPL Licence 2.0.
#
# v1.0.3 - Herve Roux
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation ; either version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses>.
#
#
####################################################################################
#
# Help : ./sde.pl -h
#

my $CheckOnly;

sub print_success {printf "\e[60G\[\e[32m  OK  \e[m]\n";}
sub print_passed {printf "\e[60G\[\e[33mPASSED\e[m]\n";}
sub print_warning {printf "\e[60G\[\e[33mWARNING\e[m]\n";}
sub print_failure {printf "\e[60G\[\e[31mFAILED\e[m]\n";}


my $ARGSTRING = '';
foreach (@ARGV) {
	if ( $_ =~ / / ){
		$ARGSTRING = $ARGSTRING." '".$_."'";
	} else {
		$ARGSTRING = $ARGSTRING." ".$_;	
	}
}
			
BEGIN{
	foreach (@ARGV) {
 		if ( $_ eq "--check" ){
 			$CheckOnly = '1';
 		}
	} 
	
	if ( $CheckOnly ){
		
		my $CheckOK = 1;
		
		sub CheckMod {
			my @parms = @_;
			for (@parms) {
				print "    * Checking for perl Module \'$_\'";
				system ("/usr/bin/perl -m$_ -e 1 > 0 2>&1");
				if ($? == 0) {print_success} else {print_failure; $CheckOK = 0;}
			}
		}
		
		print "\nChecking PERL Modules availability:\n";
		CheckMod(	'Getopt::Long',
					'SOAP::Lite',
					'Time::Local',
					'Digest::SHA1',
					'MIME::Base64',
					'Sys::Hostname',
					'Socket',
					'Net::Domain',
					'Net::Address::IP::Local',
					'Data::UUID',
					'User::pwent',
					'File::stat',
					'POSIX',
					);			
		print "\n";
		
		if ($CheckOK == 0 ){exit 1}
		
	}
}

if ( ! @ARGV ){
	print_header();
	print_usage();
	exit 0;
}
	
use strict;
use Getopt::Long;
#use SOAP::Lite ( +trace => all, maptype => {} );
use SOAP::Lite maptype => {};
use Time::Local;
use Digest::SHA1;
use MIME::Base64;
use Sys::Hostname;
use Socket;
use Net::Domain qw (hostfqdn hostdomain);
use Net::Address::IP::Local;
use Data::UUID;
use POSIX qw(strftime);


*default_nonce_generator = create_generator( "a value of ", int(1000*rand()) );

my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $ERROR_STRING='';
my @SOAP_ERRORS = ();

my $HCmessage='';
my $ShowHelp='';
my $Cust_Conf_File='';
my %Raw_SOAP_Param;

my $ng_State='';
my $ng_StateType='';
my $ng_Attempt='';
my $ng_FQDN='';
my $ng_CATEGORY='';
my $ng_GROUPASSIGNMENT='';
my $ng_HOSTADDRESS='';
my $ng_ALIAS='';
my $ng_HOSTNAME='';
my $ng_SERVICENAME='';
my $ng_HOSTGROUPNAMES='';
my $ng_SERVICEGROUPNAMES='';
my $ng_EVENTID='';
my $ng_LASTEVENTID='';
my $ng_PROBLEMID='';
my $ng_LASTPROBLEMID='';
my $ng_DATETIME='';
my $ng_OUTPUT='';
my $ng_LONG_OUTPUT='';
my $ng_DOWNTIME=0;
my $ng_DISABLE='';
my $ng_DISABLE_ACK='';
my $ng_NOTIFICATIONTYPE='';

Getopt::Long::Configure('bundling');
GetOptions ('s:s' => \$ng_State,				'state=s'            => \$ng_State,
        	't:s' => \$ng_StateType,			'type:s'             => \$ng_StateType,				
        	'n:s' => \$ng_Attempt,				'attempt:s'          => \$ng_Attempt,
        	'f:s' => \$ng_FQDN,					'fqdn:s'             => \$ng_FQDN,
        	'C:s' => \$ng_CATEGORY,				'category:s'         => \$ng_CATEGORY,
        	'G:s' => \$ng_GROUPASSIGNMENT,		'GroupAssignment:s'  => \$ng_GROUPASSIGNMENT,			
        	'a:s' => \$ng_HOSTADDRESS,			'address:s'          => \$ng_HOSTADDRESS,
        	'A:s' => \$ng_ALIAS,				'alias:s'            => \$ng_ALIAS,
        	'H:s' => \$ng_HOSTNAME,				'host:s'             => \$ng_HOSTNAME,
        	'S:s' => \$ng_SERVICENAME,			'service:s'          => \$ng_SERVICENAME,
        	'h:s' => \$ng_HOSTGROUPNAMES,		'hostgroups:s'       => \$ng_HOSTGROUPNAMES,
        	'g:s' => \$ng_SERVICEGROUPNAMES,	'servicegroups:s'    => \$ng_SERVICEGROUPNAMES,
        	'i:s' => \$ng_EVENTID,				'eventid:s'          => \$ng_EVENTID,
        	'I:s' => \$ng_LASTEVENTID,			'lasteventid:s'      => \$ng_LASTEVENTID,
        	'p:s' => \$ng_PROBLEMID,			'problemid:s'        => \$ng_PROBLEMID,
        	'P:s' => \$ng_LASTPROBLEMID,		'lastproblemid:s'    => \$ng_LASTPROBLEMID,
        	'T:s' => \$ng_DATETIME,				'time:s'             => \$ng_DATETIME,
        	'o:s' => \$ng_OUTPUT,				'output:s'           => \$ng_OUTPUT,
        	'O:s' => \$ng_LONG_OUTPUT,			'longoutput:s'       => \$ng_LONG_OUTPUT,
        	'w:s' => \$ng_DOWNTIME,				'downtime:s'         => \$ng_DOWNTIME,
        	'd:s' => \$ng_DISABLE,				'disable:s'          => \$ng_DISABLE,
        	'D:s' => \$ng_DISABLE_ACK,			'disableack:s'       => \$ng_DISABLE_ACK,
        	'N:s' => \$ng_NOTIFICATIONTYPE,		'notificationtype:s' => \$ng_NOTIFICATIONTYPE,
        	'c:s' => \$Cust_Conf_File,			'conf:s'             => \$Cust_Conf_File,
        	'HCmessage' => \$HCmessage,
        	'help' => \$ShowHelp,
        	'check' => \$CheckOnly,
        	'SOAPdata=s' =>\%Raw_SOAP_Param,
);

if ( $ShowHelp )
{
	print_help();
	exit 0;
}


if ( ! $Cust_Conf_File ){$Cust_Conf_File = "/etc/nagios/sde/sde.conf"}
if ( $ng_DATETIME ){$ng_DATETIME = timeConvert($ng_DATETIME)}


our $sde_log_file;
our $sde_uri;
our $sde_url;
our $sde_url_HC;
our $sde_ticket_link;
our $sde_ws_security;
our $sde_ws_login;
our $sde_ws_password;
our $ng_pipe;
our $ng_contact;
our $ng_set_acknowledge_on_ticket_creation;
our $ng_srv_uuid_file;
our $mailer;
our $ng_email_contact;
our $ng_max_retry;
our $ng_wait_retry;
our $ng_cust_notification;
our $sde_url_Recovery;
our $sde_url_RecoveryQuery;
our $ngsrv_addr; 
our $ngsrv_fqdn;
our $sde_timeout;
our $ng_use_hostname_as_fqdn;
our $ng_use_alias_as_fqdn;
our $send_Recovery_as_normal_event;
our $ignore_event_on_downtime;
our @filter_notification;

use User::pwent;
use File::stat;

sub Check_perm{
	my ($FileName, $User) = @_;
	
	my $fileinfo = stat($FileName);
	unless (defined $fileinfo){
		return 0;
	}
	my $NagiosUser = getpwnam("nagios");
	if ($NagiosUser){
		if ($fileinfo->uid == $NagiosUser->uid){
			return 1;
		} else {
			return 0;
		}
	}
	return 0;
}

# Load config File

if ( $CheckOnly ){print "Checking Configuration File:\n    * ";}
if ($Cust_Conf_File){
	if (-e $Cust_Conf_File){
		
		if ( $CheckOnly ){
			print "Checking Ownership";
			if (Check_perm($Cust_Conf_File, "nagios")){
				print_success;
			}else{
				print_warning;
				print "File '$Cust_Conf_File' is not owned by user 'nagios'. Consider changing permision!\n";
			}
			print "    * ";
		}
		
		if (open(TMP_Conf, "< $Cust_Conf_File")){
    		close(TMP_Conf);
    		do $Cust_Conf_File;
		} else {
			print "Cannot Read Configuration file";
			if ( $CheckOnly ){print_failure}else{print "\n";}
			exit 1;
		}
		
		if ($@) { print "Cannot parse configuration file"; if ( $CheckOnly ){print_failure}else{print "\n";} exit 1;}
		if ( $CheckOnly ){print "Configuration file loaded"; print_success}
	} else {
		print "Configuration file not found";
		if ( $CheckOnly ){print_failure}else{print "\n";}
		exit 1;
	}
}


# Setting Default Options #############################################################################
if ( ! $sde_log_file ){$sde_log_file = '';}

my $DefaultOptionCritical;
if ( $CheckOnly ){
	my $HeaderPrinted;
	if ( ! $sde_uri ){if(!$HeaderPrinted){print "Missing Mandatory Parameters in configuration file:\n";$HeaderPrinted=1;} print "    * Parameter '\$sde_uri' not found"; print_warning }
	if ( ! $sde_url ){if(!$HeaderPrinted){print "Missing Mandatory Parameters in configuration file:\n";$HeaderPrinted=1;} print "    * Parameter '\$sde_url' not found"; print_warning }
	if ( ! $sde_url_HC ){if(!$HeaderPrinted){print "Missing Mandatory Parameters in configuration file:\n";$HeaderPrinted=1;} print "    * Parameter '\$sde_url_HC' not found";print_warning }
	if ( ! $sde_url_Recovery ){if(!$HeaderPrinted){print "Missing Mandatory Parameters in configuration file:\n";$HeaderPrinted=1;} print "    * Parameter '\$sde_url_Recovery' not found"; print_warning }
} else {
	if ( ! $sde_uri ){PrintAndLogError("Missing parameter '\$sde_uri' in configuration file!\n"); $DefaultOptionCritical=1; }
	if ( ! $sde_url ){PrintAndLogError("Missing parameter '\$sde_url' in configuration file!\n"); $DefaultOptionCritical=1; }
	if ( ! $sde_url_HC ){PrintAndLogError("Missing parameter '\$sde_url_HC' in configuration file!\n"); $DefaultOptionCritical=1; }
	if ( ! $sde_url_Recovery ){PrintAndLogError("Missing parameter '\$sde_url_Recovery' in configuration file!\n"); $DefaultOptionCritical=1; }
}
if ( ! $sde_ticket_link ){$sde_ticket_link = '';}
if ( ! $sde_url_RecoveryQuery ){$sde_url_RecoveryQuery = '';}

if ( ! $sde_ws_security ){$sde_ws_security = '';}
if ( ! $sde_ws_login ){$sde_ws_login = '';}
if ( ! $sde_ws_password ){$sde_ws_password = '';}

if ( ! $ng_max_retry ){$ng_max_retry = 3;}
if ( ! $ng_wait_retry ){$ng_wait_retry = 30;}

if ( ! $ngsrv_addr ){$ngsrv_addr = Net::Address::IP::Local->public;}
if ( ! $ngsrv_fqdn ){$ngsrv_fqdn = hostfqdn();}
if ( ! $sde_timeout ){$sde_timeout = 60;}
if ( ! $ng_use_hostname_as_fqdn ){$ng_use_hostname_as_fqdn = 'no';}
if ( ! $ng_use_alias_as_fqdn ){$ng_use_alias_as_fqdn = 'no';}
if ( ! $ng_pipe ){$ng_pipe = '';}
if ( ! $mailer ){$mailer = '';}
if ( ! $ng_email_contact ){$ng_email_contact = '';}
if ( ! $ng_cust_notification ){$ng_cust_notification = -1;}
if ( ! $ng_contact ){$ng_contact = 'Supervisor';}
if ( ! $ng_set_acknowledge_on_ticket_creation ){$ng_set_acknowledge_on_ticket_creation = 'no';}
if ( ! $send_Recovery_as_normal_event ){$send_Recovery_as_normal_event = 'no';}
if ( ! $ignore_event_on_downtime ){$ignore_event_on_downtime = 'yes';}
if ( ! $ng_srv_uuid_file ){$ng_srv_uuid_file = "/etc/nagios/sde/ng_srv.uuid"}
#######################################################################################################


# get or create Nagios Server UUID
sub CheckUUIDOwner {
	if ( $CheckOnly ){
		print "    * Checking UUID file Ownership";
		if (Check_perm($ng_srv_uuid_file, "nagios")){
			print_success;
		}else{
			print_warning;
			print "File '$ng_srv_uuid_file' is not owned by user 'nagios'. Consider changing permision!\n";
		}
	}
}

my $serverUUID = '';
if ( $CheckOnly ){print "\nNagios Server UUID";}
if (-e $ng_srv_uuid_file){
	if ( open(TMP_UUID, "< $ng_srv_uuid_file") ){
		while (<TMP_UUID>)
	             {
	             $serverUUID="$_";
	             }
	    close(TMP_UUID);
	    if ( $CheckOnly ){print " - $serverUUID";print_success}
	    CheckUUIDOwner;
	} else {
		if ( $CheckOnly ){print " - "}
    	print "Cannot Read UUID File";
		if ( $CheckOnly ){print_failure}else{print "\n"}
		CheckUUIDOwner;
		exit $ERRORS{"CRITICAL"};
	}
    
} else {
	my $ug =new Data::UUID;
	$serverUUID = $ug->to_string($ug->create());
	if (open(TMP_UUID, "> $ng_srv_uuid_file")){
		print TMP_UUID "$serverUUID";
    	close(TMP_UUID);
    	if ( $CheckOnly ){print " - $serverUUID";print_success}
    	if (! Check_perm($ng_srv_uuid_file, "nagios")){`chown nagios:nagios $ng_srv_uuid_file > 0 2>1`;}
    	CheckUUIDOwner;
	} else {
    	if ( $CheckOnly ){print " - "}
    	print "Cannot Create UUID File";
    	if ( $CheckOnly ){print_failure}else{print "\n"}
    	exit $ERRORS{"CRITICAL"};
    }
}

if ( $CheckOnly ){
	print "    * fqdn - $ngsrv_fqdn\n";
	print "    * IP address - $ngsrv_addr\n\n";
	
	print "Checking nagios command pipe";
	if ( -e $ng_pipe){print_success}else{print_warning}
	print "Checking Mailer";
	my @tmp_mailer = split(/ /,$mailer);
	if ( -e $tmp_mailer[0]  ){print_success}else{print_warning}
	
	exit 0;
}
else {
	if($DefaultOptionCritical){exit 1}
}

# Setting FQDN if needed
if (lc($ng_use_hostname_as_fqdn) eq 'yes') {
	$ng_FQDN = $ng_HOSTNAME;
}
if (lc($ng_use_alias_as_fqdn) eq 'yes') {
	$ng_FQDN = $ng_ALIAS;
}
# ---

# Clean up custom Macro to blank if not implemented in nagios
if ( $ng_FQDN =~ /^\$/ && $ng_FQDN =~ /\$$/ ){$ng_FQDN=''}
if ( $ng_CATEGORY =~ /^\$/ && $ng_CATEGORY =~ /\$$/ ){$ng_CATEGORY=''}
if ( $ng_GROUPASSIGNMENT =~ /^\$/ && $ng_GROUPASSIGNMENT =~ /\$$/ ){$ng_GROUPASSIGNMENT=''}
if ( $ng_DISABLE =~ /^\$/ && $ng_DISABLE =~ /\$$/ ){$ng_DISABLE='no'}
if ( $ng_DISABLE_ACK =~ /^\$/ && $ng_DISABLE_ACK =~ /\$$/ ){$ng_DISABLE_ACK='no'}
if ( $ng_NOTIFICATIONTYPE =~ /^\$/ && $ng_NOTIFICATIONTYPE =~ /\$$/ ){$ng_NOTIFICATIONTYPE='no'}

# Nagios V2 Backward compatibility - Macro Clean up
if ( $ng_EVENTID =~ /^\$/ && $ng_EVENTID =~ /\$$/ ){$ng_EVENTID=''}
if ( $ng_LASTEVENTID =~ /^\$/ && $ng_LASTEVENTID =~ /\$$/ ){$ng_LASTEVENTID=''}
if ( $ng_PROBLEMID =~ /^\$/ && $ng_PROBLEMID =~ /\$$/ ){$ng_PROBLEMID=''}
if ( $ng_LASTPROBLEMID =~ /^\$/ && $ng_LASTPROBLEMID =~ /\$$/ ){$ng_LASTPROBLEMID=''}
if ( $ng_LONG_OUTPUT =~ /^\$/ && $ng_LONG_OUTPUT =~ /\$$/ ){$ng_LONG_OUTPUT=''}
if ( $ng_HOSTGROUPNAMES =~ /^\$/ && $ng_HOSTGROUPNAMES =~ /\$$/ ){$ng_HOSTGROUPNAMES=''}
if ( $ng_SERVICEGROUPNAMES =~ /^\$/ && $ng_SERVICEGROUPNAMES =~ /\$$/ ){$ng_SERVICEGROUPNAMES=''}

# Primary Filter of Events ********************************************************
# *********************************************************************************

# CALL Send HC message
if ( $HCmessage ){
	exit SendHCmessage();
}

# Exit on SOFT Event
if ($ng_StateType eq 'SOFT'){exit 0};

# Filter notification Type
if ($ng_NOTIFICATIONTYPE eq 'ACKNOWLEDGEMENT'){
	if (lc($ng_set_acknowledge_on_ticket_creation) eq 'yes' ) {
		exit 0;
	}
}
# Filter notification Type Array loop
foreach (@filter_notification) {
	if ( $_ eq $ng_NOTIFICATIONTYPE ){
		exit 0;
	}
}

# Log Call From Here
logEvent("<CALL> ".$0.$ARGSTRING);

# Filter if the Event Handler is disable with custom Macro
if (lc($ng_DISABLE) eq 'yes'){
	logEvent("<FILTER> "."SDE Event Handler is disable using '_DISABLE_SDE' custom macro. Last Event Filtered.");
	exit 0;
}

# Filter DownTime If needed
if ($ng_DOWNTIME>0 and lc($ignore_event_on_downtime) eq 'yes'){
	logEvent("<FILTER> "."$ng_HOSTNAME is in downtime. Last Event Filtered.");
	exit 0;
}

# *********************************************************************************
# *********************************************************************************

# Checking mandatory Fields #####################################################
if ( ! $ng_FQDN ){PrintAndLogError("No FQDN Specified\n"); exit 1;}
if ( ! $ng_DATETIME ){PrintAndLogError("No valid Date Specified\n"); exit 1;}
#################################################################################


# Compute Severity		
my $sde_severity = 0;
if($ng_State eq "CRITICAL" ){ $sde_severity = 1;}
if($ng_State eq "DOWN" ){ $sde_severity = 1;}
if($ng_State eq "WARNING" ){ $sde_severity = 2;}
if($ng_State eq "UNREACHABLE" ){ $sde_severity = 3;}
if($ng_State eq "UNKNOWN" ){ $sde_severity = 3;}

my @Raw_SOAP_data;
# Build of the Generic SOAP data Param 
while ( my ($key, $value) = each(%Raw_SOAP_Param) ) {
	if ( $value =~ /^\$/ && $value =~ /\$$/ ){$value=''}
	push(@Raw_SOAP_data, SOAP::Data->name($key => $value));
}

# DO the ACTUAL WEB SERVICES CALL ------------------------------------------------
sub CallProperWebService {
	if($ng_State eq "OK" or $ng_State eq "UP"){
		if (lc($send_Recovery_as_normal_event) eq 'yes') {
			return(SendEventMessage(@Raw_SOAP_data));
		} else {
			if ($sde_url_RecoveryQuery){
				# Query First Before Recovery
				return(QueryForRecoveryMessage(@Raw_SOAP_data));
			}else{
				# Recovery
				return(SendRecoveryMessage(@Raw_SOAP_data));
			}
		}
	}else{
		return(SendEventMessage(@Raw_SOAP_data));
	}
}

my $RETRY_COUNTER;
for ($RETRY_COUNTER = 1; $RETRY_COUNTER <= $ng_max_retry ; $RETRY_COUNTER++){
	
	if(CallProperWebService() == $ERRORS{"OK"}){last;}
	
	if ($RETRY_COUNTER < $ng_max_retry){
		print "Retrying in $ng_wait_retry Seconds...\n";
		sleep $ng_wait_retry;
	} else {
		print "Done trying, Sending panic email...\n";
		
			my $mailbody = "* Call Details \n\n";
			$mailbody = $mailbody."	FQDN : $ng_FQDN \n";
			$mailbody = $mailbody."	CATEGORY : $ng_CATEGORY \n";
			$mailbody = $mailbody."	GROUPASSIGNEMENT : $ng_GROUPASSIGNMENT \n";
			$mailbody = $mailbody."	HOSTNAME : $ng_HOSTNAME \n";
			$mailbody = $mailbody."	HOSTADDRESS : $ng_HOSTADDRESS \n";
			$mailbody = $mailbody."	ALIAS : $ng_ALIAS \n";
			$mailbody = $mailbody."	SERVICENAME : $ng_SERVICENAME \n";
			$mailbody = $mailbody."	HOST GROUP NAMES : $ng_HOSTGROUPNAMES \n";
			$mailbody = $mailbody."	SERVICE GROUP NAMES : $ng_SERVICEGROUPNAMES \n";
			$mailbody = $mailbody."	Severity : $sde_severity \n";
			$mailbody = $mailbody."	State : $ng_State \n";
			$mailbody = $mailbody."	StateType : $ng_StateType \n";
			$mailbody = $mailbody."	Attempt : $ng_Attempt \n";
			$mailbody = $mailbody."	EVENTID : $ng_EVENTID \n";
			$mailbody = $mailbody."	LASTEVENTID : $ng_LASTEVENTID \n";
			$mailbody = $mailbody."	PROBLEMID : $ng_PROBLEMID \n";
			$mailbody = $mailbody."	LASTPROBLEMID : $ng_LASTPROBLEMID \n";
			$mailbody = $mailbody."	GMT DATE/TIME : $ng_DATETIME\n";
			$mailbody = $mailbody."	OUTPUT : $ng_OUTPUT \n";
			$mailbody = $mailbody."	LONG OUTPUT : $ng_LONG_OUTPUT \n";

		SendFallBacKeMail($ERROR_STRING,$mailbody)
	}
}

exit $ERRORS{"OK"};



# Subs #####################################################################################################

# ---
# Return the function (WS operation) Name
sub GetWSop {
	my ($string2parse) = @_;
	
	my $startParam = index($string2parse,"?");
	if ($startParam >= 0){
		my $startOP = index($string2parse,"?op=",$startParam);
		if ($startOP == -1){$startOP = index($string2parse,"&op=",$startParam)}
		if ( $startOP >= 0){
			my $stopOP = index($string2parse,"&",$startOP+1);
			if ($stopOP >= 0){
				return substr ($string2parse,$startOP+4,$stopOP-$startOP-4);
			}else{
				return substr ($string2parse,$startOP+4);	
			}
		}else{
			return substr ($string2parse,rindex($string2parse,"/",$startParam)+1,rindex($string2parse,".",$startParam)-rindex($string2parse,"/",$startParam)-1);
		}
	}else{
		return substr ($string2parse,rindex($string2parse,"/")+1,rindex($string2parse,".")-rindex($string2parse,"/")-1);
	}
}

# ----
# $error_string = Error String 2 Log
sub logEvent{
	my ($_error_string) = @_;

	# Activate Logging if needed
	if (! $sde_log_file eq '') {
		if ( ! open LOG_FILE ,">>$sde_log_file"){
			print "can't open $sde_log_file: $!\n";
		} else {
			# Do actual right
			if ( $_error_string !~ /\n$/){$_error_string = $_error_string."\n";}
			print $_error_string;
			print LOG_FILE "[".strftime('%b %d %H:%M:%S',localtime)."] ".$_error_string;
			close LOG_FILE;
		}
	}
}

# ----
# $error_string = Error String 2 Log
# $LogInComments = Log in Nagios Comment
sub PrintAndLogError {
	my ($_error_string, $LogInComments) = @_;
	
	if ( $_error_string !~ /\n$/){$_error_string = $_error_string."\n";}
	#print $_error_string; # now done in log
	logEvent($_error_string);
	if($LogInComments){set_comment($_error_string)};
	$ERROR_STRING = $_error_string;
}


# ----
# handle Soap Errors
sub soapError {
	my $soap = shift;
	my $res = shift;
	my $err;
	eval {$err = $res->faultstring};
	if( $err ) {
		push( @SOAP_ERRORS, "<SOAP ERROR> $err");
	}
	else {
		my $transerr;
		eval {$transerr = $soap->transport->status};
		if ($transerr){
			push( @SOAP_ERRORS, "<TRANSPORT ERROR> $transerr");
		}
		else {
			push( @SOAP_ERRORS, "<Unknown ERROR> ");
		}
	}
	return new SOAP::SOM;
}



# ----
#
sub QueryForRecoveryMessage {
	
	my $soap = SOAP::Lite
			-> on_fault( \&soapError )
		    -> uri($sde_uri)
		    -> on_action( sub { join '/', 'http://bmc.com/webservices', $_[1] } )
		    -> proxy($sde_url_RecoveryQuery, timeout => $sde_timeout);

	my $methodName = GetWSop($sde_url_RecoveryQuery); 
	my $method = SOAP::Data->name($methodName)
	    ->attr({xmlns => 'http://bmc.com/webservices/'});
	
	# ref http://www.wlp-systems.de/soap-lite-and-ws-security.html
	# ------------------------------------------------------------
	my $header;
	if (lc($sde_ws_security) eq 'yes') {
		$header = ws_authen($sde_ws_login,$sde_ws_password);
	}
	
	my @params = ( $header,
	               SOAP::Data->name(ServerUUID => $serverUUID),
	               SOAP::Data->name(AlertID => $ng_LASTPROBLEMID),
	               @_,
	);
	
	my $result;
	$result = $soap->call($method => @params);
	
	if (scalar( @SOAP_ERRORS ) > 0){
		PrintAndLogError("<$methodName> ".pop(@SOAP_ERRORS));
		return $ERRORS{"CRITICAL"};
	}
	else
	{
		unless ($result->fault) {
			
			if ($result->valueof('//HasError') eq "true"){
				PrintAndLogError("<$methodName> <Responce Has Error> ".$result->valueof('//ErrorMessage'));
				#print $result->valueof('//StackTrace')."\n";
				return $ERRORS{"CRITICAL"};
			}
			else {
				# Loop Incident List and call the recovery message for each incident
				my @Incidents = $result->valueof('//IncidentList/Incident');
				logEvent("<$methodName> <Result> ".scalar(@Incidents)." element(s) returned.");
				foreach my $Incident (@Incidents)
				{
					logEvent("<$methodName> Sending Recovery Message for SDE Incident ".$Incident->{'IncidentNumber'});
					SendRecoveryMessage(SOAP::Data->name(IncidentNumber => $Incident->{'IncidentNumber'}), @_);
				}
			}
			 
		} else {
			PrintAndLogError("<$methodName> <".$result->faultcode."> ".$result->faultstring);
		    return $ERRORS{"CRITICAL"};
		}
	}
	return $ERRORS{"OK"};
	
} # QueryForRecoveryMessage

# *************************************************************************************************************

# ----
#
sub SendRecoveryMessage {
	
	my $soap = SOAP::Lite
			-> on_fault( \&soapError )
		    -> uri($sde_uri)
		    -> on_action( sub { join '/', 'http://bmc.com/webservices', $_[1] } )
		    -> proxy($sde_url_Recovery, timeout => $sde_timeout);

	my $methodName = GetWSop($sde_url_Recovery);
	my $method = SOAP::Data->name($methodName)
	    ->attr({xmlns => 'http://bmc.com/webservices/'});
	
	# ref http://www.wlp-systems.de/soap-lite-and-ws-security.html
	# ------------------------------------------------------------
	my $header;
	if (lc($sde_ws_security) eq 'yes') {
		$header = ws_authen($sde_ws_login,$sde_ws_password);
	}
	
	my @params = ( $header,
	               SOAP::Data->name(ServerUUID => $serverUUID),
	               SOAP::Data->name(alertID => $ng_LASTPROBLEMID),
	               @_,
	);
	
	my $result;
	$result = $soap->call($method => @params);
	
	if (scalar( @SOAP_ERRORS ) > 0){
		PrintAndLogError("<$methodName> ".pop(@SOAP_ERRORS));
		return $ERRORS{"CRITICAL"};
	}
	else
	{
		unless ($result->fault) {
			
			if ($result->valueof('//HasError') eq "true"){
				PrintAndLogError("<$methodName> <Responce Has Error> ".$result->valueof('//ErrorMessage'));
				#print $result->valueof('//StackTrace')."\n";
				return $ERRORS{"CRITICAL"};
			}
			else {
				my $sde_caseIDs = '';
				#$sde_caseID = $result->valueof('//IncidentList/Incident/IncidentNumber');
				my @Incidents = $result->valueof('//IncidentList/Incident');
				foreach my $Incident (@Incidents)
				{
				   $sde_caseIDs=$sde_caseIDs." ".$Incident->{'IncidentNumber'};
				}
			    logEvent("<$methodName> SDE Incident(s)".$sde_caseIDs." set to Resolved");
			}
			 
		} else {
			PrintAndLogError("<$methodName> <".$result->faultcode."> ".$result->faultstring);
		    return $ERRORS{"CRITICAL"};
		}
	}
	return $ERRORS{"OK"};
	
} # SendRecoveryMessage

# *************************************************************************************************************

sub SendHCmessage {
	
	my $soap = SOAP::Lite
			-> on_fault( \&soapError )
			-> uri($sde_uri)
		    -> on_action( sub { join '/', 'http://bmc.com/webservices', $_[1] } )
		    -> proxy($sde_url_HC, timeout => $sde_timeout);
		    
	my $methodName = GetWSop($sde_url_HC);
	my $method = SOAP::Data->name($methodName)
	    ->attr({xmlns => 'http://bmc.com/webservices/'});
	
	# ref http://www.wlp-systems.de/soap-lite-and-ws-security.html
	# ------------------------------------------------------------
	my $header;
	if (lc($sde_ws_security) eq 'yes') {
		$header = ws_authen($sde_ws_login,$sde_ws_password);
	}
	
	my @params = ( $header,
	               SOAP::Data->name(ServerUUID => $serverUUID),
	               @_,
	);
	
	my $result;
	$result = $soap->call($method => @params);
	
	if (scalar( @SOAP_ERRORS ) > 0){
		PrintAndLogError("<$methodName> ".pop(@SOAP_ERRORS));
		return $ERRORS{"CRITICAL"};
	}
	else
	{
		unless ($result->fault) {
			
			if ($result->valueof('//HasError') eq "true"){
				PrintAndLogError("<$methodName> <Responce Has Error> ".$result->valueof('//ErrorMessage'));
				#print $result->valueof('//StackTrace')."\n";
				return $ERRORS{"CRITICAL"};
			}
			else {
				my $CI_Name = $result->valueof('//CI_x0020_AssemblyList/CI_x0020_Assembly/CIAssemblyID');
				print join ', ', "CI Name", $CI_Name;
				print "\n";
			}
			 
		} else {
			PrintAndLogError("<$methodName> <".$result->faultcode."> ".$result->faultstring);
		    return $ERRORS{"CRITICAL"};
		}
	}
	return $ERRORS{"OK"};
	
} # SendHCmessage

#******************************************************************************************

sub SendEventMessage {
	
	my $soap = SOAP::Lite
			-> on_fault( \&soapError )
		    -> uri($sde_uri)
		    -> on_action( sub { join '/', 'http://bmc.com/webservices', $_[1] } )
		    -> proxy($sde_url, timeout => $sde_timeout);

	my $methodName = GetWSop($sde_url);
	my $method = SOAP::Data->name($methodName)
	    ->attr({xmlns => 'http://bmc.com/webservices/'});
	
	# ref http://www.wlp-systems.de/soap-lite-and-ws-security.html
	# ------------------------------------------------------------
	my $header;
	if (lc($sde_ws_security) eq 'yes') {
		$header = ws_authen($sde_ws_login,$sde_ws_password);
	}
	
	#my $description = '';
	#if (!$ng_SERVICENAME){
	#	$description = "[$ng_HOSTNAME] $ng_OUTPUT"
	#}
	#else{
	#	$description = "[$ng_HOSTNAME][$ng_SERVICENAME] $ng_OUTPUT"
	#}
	
	my @params = ( $header,
	               SOAP::Data->name(Output =>  $ng_OUTPUT),
	               SOAP::Data->name(Hostname => $ng_HOSTNAME),
	               SOAP::Data->name(IPaddress => $ng_HOSTADDRESS),
	               SOAP::Data->name(ServiceName => $ng_SERVICENAME),
	               SOAP::Data->name(Severity => $sde_severity),
	               SOAP::Data->name(EventID => $ng_EVENTID),
	               SOAP::Data->name(LastEventID => $ng_LASTEVENTID),
	               SOAP::Data->name(AlertID => $ng_PROBLEMID),
	               SOAP::Data->name(LastAlertID => $ng_LASTPROBLEMID),
	               _typeless("DateTime", $ng_DATETIME),
	               SOAP::Data->name(SourceID => 'Nagios'),
	               SOAP::Data->name(LongOutput => $ng_LONG_OUTPUT),
	               SOAP::Data->name(ServerIP => $ngsrv_addr),
	               SOAP::Data->name(ServerFQDN => $ngsrv_fqdn),
	               SOAP::Data->name(GroupAssignment => $ng_GROUPASSIGNMENT),
	               SOAP::Data->name(ServerUUID => $serverUUID),
	               SOAP::Data->name(HostAlias => $ng_ALIAS),
	               SOAP::Data->name(HostGroupNames => $ng_HOSTGROUPNAMES),
	               SOAP::Data->name(ServiceGroupNames => $ng_SERVICEGROUPNAMES),
	               SOAP::Data->name(fqdn => $ng_FQDN),
	               SOAP::Data->name(Category => $ng_CATEGORY),
	               SOAP::Data->name(State => $ng_State),
	               SOAP::Data->name(DownTime => $ng_DOWNTIME),
	               @_,
	);
	
	#my $result = $soap -> CreateIncident('Test Incident', 'BOXSRV', 'BOXNAG', 'Nagios');
	
	#my $result;
	#eval { $result = $soap->call($method => @params); };
	#if ($@) {
	#	PrintAndLogError("<$methodName> <ERROR> ".$@, 1);
	#	return $ERRORS{"CRITICAL"};
	#}
	
	my $result;
	$result = $soap->call($method => @params);
	
	if (scalar( @SOAP_ERRORS ) > 0){
		PrintAndLogError("<$methodName> ".pop(@SOAP_ERRORS), 1);
		return $ERRORS{"CRITICAL"};
	}
	else
	{
		unless ($result->fault) {
			
			if ($result->valueof('//HasError') eq "true"){
				PrintAndLogError("<$methodName> <Responce Has Error> ".$result->valueof('//ErrorMessage'), 1);
				#print $result->valueof('//StackTrace')."\n";
				return $ERRORS{"CRITICAL"};
			}
			else {
				my $sde_caseID = '';
				#$sde_caseID = $result->valueof('//IncidentList/Incident/IncidentNumber');
				my @Incidents = $result->valueof('//IncidentList/Incident');
				foreach my $Incident (@Incidents)
				{
				   $sde_caseID=$sde_caseID." ".$Incident->{'IncidentNumber'};
				}						
			    logEvent("<$methodName> SDE Incident Number".$sde_caseID." created or updated");
			    
			    my $ng_cust_msg = "An incident was opened in SDE with id: ";
			    if ($sde_ticket_link){
			    	$ng_cust_msg=$ng_cust_msg."<A HREF=\'$sde_ticket_link$sde_caseID\' target=\"_blank\">$sde_caseID</A>\n";	
			    } else {
			    	$ng_cust_msg=$ng_cust_msg."$sde_caseID\n";
			    } 
			    
			    # Set ACK or Comment with Nagios CMD
			    if (lc($ng_set_acknowledge_on_ticket_creation) eq 'yes' ) {
			    	if (lc($ng_DISABLE_ACK) eq 'yes'){
						logEvent("<Set_Acknowledge> "."Acknowledge is disable using '_DISABLE_ACK' custom macro. Simple comment will be set instead.");
						set_comment($ng_cust_msg);
					} else {
			    		set_acknowledge($ng_cust_msg);
					}
			    } else {
			    	set_comment($ng_cust_msg);
			    }
			}
			 
		} else {
			PrintAndLogError("<$methodName> <".$result->faultcode."> ".$result->faultstring, 1);
		    return $ERRORS{"CRITICAL"};
		}
	}
	return $ERRORS{"OK"};
	
} # SendEventMessage



sub SOAP::Transport::HTTP::Client::get_basic_credentials {
   return 'username' => 'password';
}

sub xml_quote {
    my ($value) = @_;
    $value =~ s/&/&amp;/;
    $value =~ s/</&lt;/;
    $value =~ s/>/&gt;/;
    $value;
}

sub _complex_type {
    my ($name,@childs) = @_;
    my $data = SOAP::Data->new( name => $name );
    $data->value( \SOAP::Data->value(@childs));
    #$data->value( \SOAP::Data->value($_[1], $_[2], $_[3], $_[4]));
    $data;
}

sub _typeless {
    my ($name,$value) = @_;
    my $data = SOAP::Data->new( name => $name );

    $value = xml_quote($value);

    $data->value( $value );
    $data->type( "" );
    $data;
}

sub timestamp {
    my ($sec,$min,$hour,$mday,$mon,$year,undef,undef,undef) = gmtime(time);
    $mon++;
    $year = $year + 1900;
    return sprintf("%04d-%02d-%02dT%02d:%02d:%02dZ",$year,$mon,$mday,$hour,$min,$sec);
}
sub timeConvert {
	my ($TimeParam) = @_;
    my ($sec,$min,$hour,$mday,$mon,$year,undef,undef,undef) = gmtime($TimeParam);
    $mon++;
    $year = $year + 1900;
    return sprintf("%04d-%02d-%02dT%02d:%02d:%02dZ",$year,$mon,$mday,$hour,$min,$sec);
}

sub create_generator {
    my ($name,$start_with) = @_;
    my $i = $start_with;
    return sub {  $name . ++$i; };
}

sub ws_authen {
    my($username,$passwort,$nonce_generator) = @_;
    if(!defined($nonce_generator)) {
        $nonce_generator = \&default_nonce_generator;
    }
    my $nonce = $nonce_generator->();
    my $timestamp = timestamp();

    my $pwDigest =  Digest::SHA1::sha1( $nonce . $timestamp . $passwort );
    my $passwortHash = MIME::Base64::encode_base64($pwDigest,"");
    my $nonceHash = MIME::Base64::encode_base64($nonce,"");

    my $auth = SOAP::Header->new( name => "wsse:Security" );
    $auth->attr( {
        "xmlns:wsse" => "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd",
        "xmlns:wsu"  => "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd",
    });
    $auth->mustUnderstand(1);

    $auth->value( \SOAP::Data->value(
         _complex_type ( "wsse:UsernameToken",
            _typeless("wsse:Username",$username),
            _typeless("wsse:Password",$passwort),
           # _typeless("wsse:Password",$passwortHash)->attr({
           #     "Type" => "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordDigest"
           # }),
           # _typeless("wsse:Nonce",$nonceHash),
           # _typeless("wsu:Created",$timestamp),
        )
       )
    );
    $auth;
}

# Here we acknowledge the problem in Nagios
sub set_acknowledge
{
	my ($NG_ACK_PARAM) = @_;
    my $NG_ACK_MSG="";
    
    if (-e $ng_pipe)
	{ 
	    if (open(NG_CMD, ">> $ng_pipe")){
	    
		    my $now = time();
		    if (!$ng_SERVICENAME) {
		       # ACKNOWLEDGE_HOST_PROBLEM;<host_name>;<sticky>;<notify>;<persistent>;<author>;<comment>
		       $NG_ACK_MSG="[$now] ACKNOWLEDGE_HOST_PROBLEM;$ng_HOSTNAME;1;1;1;$ng_contact;";
		    }
		    else {
		       # ACKNOWLEDGE_SVC_PROBLEM;<host_name>;<service_description>;<sticky>;<notify>;<persistent>;<author>;<comment>
		       $NG_ACK_MSG="[$now] ACKNOWLEDGE_SVC_PROBLEM;$ng_HOSTNAME;$ng_SERVICENAME;1;1;1;$ng_contact;";
		       
		    }
		    
		    if (!$NG_ACK_PARAM){
		    	$NG_ACK_MSG=$NG_ACK_MSG."(blank)";
		    } else {
		    	$NG_ACK_MSG=$NG_ACK_MSG.$NG_ACK_PARAM
		    }
		    logEvent("<Set_Acknowledge> $NG_ACK_MSG");
			print NG_CMD "$NG_ACK_MSG";
		    close(NG_CMD);
	    }else{
	    	logEvent("<Set_Acknowledge> <ERROR> Cannot Open $ng_pipe for writing");
	    }
	}
}



# Here we add only a comment on the problem in Nagios in case we have trouble opening an incident in SDE

sub set_comment
{
	my ($NG_COMMENT_PARAM) = @_;
    my $NG_COMMENT_MSG='';
 	 
    if (-e $ng_pipe)
	{ 
	    if (open(NG_CMD, ">> $ng_pipe")){
	  
		    my $now = time();
		    if (!$ng_SERVICENAME)
		    {
		       # ADD_HOST_COMMENT;<host_name>;<persistent>;<author>;<comment>
		       $NG_COMMENT_MSG="[$now] ADD_HOST_COMMENT;$ng_HOSTNAME;1;$ng_contact;";
		    } else {
		       # ADD_SVC_COMMENT;<host_name>;<service_description>;<persistent>;<author>;<comment>
		       $NG_COMMENT_MSG="[$now] ADD_SVC_COMMENT;$ng_HOSTNAME;$ng_SERVICENAME;1;$ng_contact;";
		    }
		       
		    if (!$NG_COMMENT_PARAM){
		    	$NG_COMMENT_MSG=$NG_COMMENT_MSG."(blank)";
		    } else {
		    	$NG_COMMENT_MSG=$NG_COMMENT_MSG.$NG_COMMENT_PARAM
		    }
		    logEvent("<Set_Comment> $NG_COMMENT_MSG");
			print NG_CMD "$NG_COMMENT_MSG";
		    close(NG_CMD);
	    }else{
	    	logEvent("<Set_Comment> <ERROR> Cannot Open $ng_pipe for writing");
	    }
	}
}


sub SendFallBacKeMail
{
	my ($ng_error, $ng_body) = @_;
	
	my $fullbody = "$ng_error\n";
	$fullbody = $fullbody."Message from Nagios:\n";
	$fullbody = $fullbody."-------------------------------------------------------------------\n";
	$fullbody = $fullbody."$ng_body\n";
	
	
	if ($ng_email_contact){
		SendManuaLeMail($fullbody)
	}
	if ( $ng_cust_notification >= 0 && $ng_cust_notification  <= 7 ){
		Send_cust_notification($fullbody);
	}
}

sub SendManuaLeMail
{
	
	my ($ng_body) = @_;
	if (open(MAIL, "|$mailer")){
	 
		# Mail Header
		print MAIL "To: $ng_email_contact\n";
		print MAIL "Subject: [Nagios] SDE not accessible. Cannot create an incident!\n\n";
		
		# Mail Body
		logEvent("<Send_panic_eMail> To: $ng_email_contact");
		print MAIL "$ng_body\n";
		close(MAIL);
	} else {
		logEvent("<Send_panic_eMail> <ERROR> Cannot Open $mailer for writing");
	}
}

sub Send_cust_notification
{
	my ($NG_COMMENT_PARAM) = @_;
    my $NG_COMMENT_MSG='';
    
    if ($NG_COMMENT_PARAM){
	    if ($ng_pipe)
		{ 
		    if (open(NG_CMD, ">> $ng_pipe")){
		    
			    my $now = time();
			    if (!$ng_SERVICENAME)
			    {
			       # SEND_CUSTOM_HOST_NOTIFICATION;<host_name>;<options>;<author>;<comment>
			       $NG_COMMENT_MSG="[$now] SEND_CUSTOM_HOST_NOTIFICATION;$ng_HOSTNAME;$ng_cust_notification;$ng_contact;";
			    } else {
			       # SEND_CUSTOM_SVC_NOTIFICATION;<host_name>;<service_description>;<options>;<author>;<comment>
			       $NG_COMMENT_MSG="[$now] SEND_CUSTOM_SVC_NOTIFICATION;$ng_HOSTNAME;$ng_SERVICENAME;$ng_cust_notification;$ng_contact;";
			    }
			       
			    $NG_COMMENT_MSG=$NG_COMMENT_MSG.$NG_COMMENT_PARAM;	
			    
			    logEvent("<Send_cust_notification> $NG_COMMENT_MSG");
				print NG_CMD "$NG_COMMENT_MSG";
			    close(NG_CMD);
			}else{
	    		logEvent("<Send_cust_notification> <ERROR> Cannot Open $ng_pipe for writing");
	    	}
		}
    }
}

sub print_usage
{
	print "\nUsage: sde.pl -f|--fqdn <fqdn> -H|--host <host> -S|--service <service> -s|--state <nagios host/service current state> -t|--type <nagios state type> -n|--attempt <nagios current host/service check retry> -C|--category <SDE category> -G|--GroupAssignment <SDE Group Assignment> -a|--address <host address> -A|--alias <host alias> -h|--hostgroups <host groups> -g|--servicegroups <service groups> -p|--problemid <nagios problem ID> -P|--lastproblemid <nagios last problem ID> -T|--time <Nagios time stamp in time_t format> -o|--output <nagios output> -O|--longoutput <nagios long output> -c|--conf <configuration file>\n";
	print "or\n";
	print "Usage: sde.pl --HCmessage to send Health check.\n";
	print "or\n";
	print "Usage: sde.pl --check to run checks.\n";
	print "or\n";
	print "Usage: sde.pl --help for help.\n\n";
}

sub print_header
{
	print "BMC Service Desk Express Nagios Connector\n";
	print "Copyright (c) 2010 GroupeOpen - http://www.open-groupe.com/\n";
	print "Developed by: Herve Roux under GPL License 2.0.\n";
}

sub print_help
{
	print_header();
    print_usage();
    print "   -f, --fqdn                     * Fully qualified domain name of the host (mandatory)\n";
    print "                                       Nagios Custom HOST Macro '_FQDN'\n";
    print "                                       The FQDN will be used in SDE to match with a CI\n";
    print "   -H, --host                     * Host causing the incident from Nagios (mandatory)\n";
    print "   -S, --service                    Service causing the incident from Nagios\n";
    print "   -a, --address                    Host Address in Nagios\n";
    print "   -A, --alias                      Host Alias in Nagios\n";
    print "   -h, --hostgroups                 Host Groups in Nagios\n";
    print "   -g, --servicegroups              Service Groups in Nagios\n";
    print "\n";
    print "   -C, --category                 * SDE Category (mandatory)\n";
    print "                                       Nagios Custom HOST & SERVICE Macro '_SDE_CATEGORY'\n";
    print "                                       The Category will be used in SDE to map with Services\n";
    print "   -G, --GroupAssignment            SDE Group Assignment\n";
    print "                                       Nagios Custom HOST & SERVICE Macro '_SDE_GROUPASSIGNMENT'\n";
    print "                                       The Group Assignment will override the Category mapping\n";
    print "                                       directive in SDE. As good practice, manage Group Assignment\n";
    print "                                       Mapping centrally from SDE using Categories. This parameter\n";
    print "                                       is a commodity feature that should be used to manage exceptions.\n";
    print "\n";
    print "   -s, --state=state              * State from Nagios (mandatory)\n";
    print "                                       UP = Host up - will send a Recovery Message\n";
    print "                                       DOWN = Host down - Mapped to severity 1\n";
    print "                                       UNREACHABLE = Host unreachable - Mapped to severity 3\n";
    print "                                       OK = Service ok - will send a Recovery Message\n";
    print "                                       WARNING = Service Warning - Mapped to severity 2\n";
    print "                                       CRITICAL = Service critical- Mapped to severity 1\n";
    print "                                       UNKNOWN = Service unknown - Mapped to severity 3\n";
    print "\n";
    print "   -t, --type                     * State Type - SOFT|HARD (mandatory)\n";
    print "   -n, --attempt                    Current host|service check retry (optional)\n";
    print "   -p, --problemid                * Nagios problem ID (mandatory)\n";
    print "   -P, --lastproblemid            * Nagios last problem ID (mandatory)\n";
    print "   -i, --eventid                    Nagios event ID\n";
    print "   -I, --lasteventid                Nagios last event ID\n";
    print "   -w, --downtime                   Nagios downtime depth\n";
    print "\n";
    print "   -d, --disable                    To disable the event, set '_DISABLE_SDE' to 'yes'.\n";
    print "   -D, --disableack                 To disable acknowledge, set '_DISABLE_ACK' to 'yes'.\n";
    print "\n";
    print "   -T, --time                     * Nagios time stamp in time_t format (mandatory)\n";
    print "   -o, --output                     Nagios text output from the last check\n";
    print "   -O, --longoutput                 Nagios full text output from the last check\n";
    print "\n";
	print "   -N, --notificationtype           Nagios type of notification that is being sent.\n";
	print "                                       => Used when CALLED by Notification\n";
    print "\n";
    print "   --SOAPdata                       SOAP data Pass-through\n";
    print "                                       Allow to pass additional nagios macro to SDE web services\n";
    print "                                       Format: --SOAPdata Key=Value --SOAPdata Key2=Value2 ...\n";
    print "\n";
    print "   -c, --conf                       Configuration File to use\n";
    print "                                       Default is '/etc/nagios/sde/sde.conf'\n";
    print "\n";
    print "   --HCmessage                      Send a Health check message to the SDE Server.\n";
    print "\n";
    print "   --check                          Run check list\n";
    print "\n";
    print "   --help                           Short help message\n";
    print "\n";
}
    
    
    